from flask import current_app
from importlib import import_module
from polzybackend.policy import Policy
from polzybackend.models import Activity


def import_class(name):
    #
    # imports class from string 'name' 
    #

    components = name.split('.')
    module_name = '.'.join(components[:-1])
    mod = import_module(module_name)
    cls = getattr(mod, components[-1])
    return cls


def get_policy_class():
    #
    # returns policy class
    #

    if current_app.config.get('CLASSNAME_POLICY'):
        return import_class(current_app.config.get('CLASSNAME_POLICY'))

    return Policy


def get_activity_class(activity_type_class):
    #
    # returns activity class
    #

    if current_app.config.get(activity_type_class):
        return import_class(current_app.config.get(activity_type_class))

    return Activity


def get_all_stages():
    #
    # returns list of all stages
    #

    if current_app.config.get('METHOD_GET_STAGES'):
        lClass = import_class(current_app.config.get('METHOD_GET_STAGES'))()
        return lClass.getAllStages
        # return import_class(current_app.config.get('METHOD_GET_STAGES'))

    raise Exception('Method to Get Stages NOT defined')


def get_permissions(user):
    #
    # return user's permissions
    #

    if current_app.config.get('METHOD_GET_PERMISSIONS'):
        permissions_method = import_class(current_app.config['METHOD_GET_PERMISSIONS'])
        return permissions_method(user)

    # default method
    return user.get_permissions()




from base import db


# 
# Attributes Values
#

class PolicyValue(db.Model):
    __tablename__ = 'policies'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    effective_date = db.Column(db.Date)
    status = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('policy', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class PolicyAttributes(db.Model):
    __tablename__ = 'policy_attributes'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))
    attribute_3 = db.Column(db.String(32))
    attribute_4 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('policy_attributes', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class PartnerValues(db.Model):
    __tablename__ = 'partners'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    is_person = db.Column(db.Boolean, nullable=False)
    first_name = db.Column(db.String(64))
    last_name = db.Column(db.String(64))
    middle_name = db.Column(db.String(64))
    birth_date = db.Column(db.Date)
    company_name = db.Column(db.String(64))
    address = db.Column(db.String(128))
    city = db.Column(db.String(64))
    country = db.Column(db.String(64))
    postal_code = db.Column(db.String(16))
    primary_email = db.Column(db.String(64))
    secondary_email = db.Column(db.String(64))
    primary_phone = db.Column(db.String(16))
    secondary_phone = db.Column(db.String(16))
    risk_group = db.Column(db.String(64))
    current_occupation = db.Column(db.String(64))
    current_occupation_from = db.Column(db.Date)
    previous_ocupation = db.Column(db.String(64))
    sports = db.Column(db.String(64))
    health_conditions = db.Column(db.String(256))
    
    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('partner', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class InsuredPersonAttributes(db.Model):
    __tablename__ = 'insured_person_attributes'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))
    attribute_3 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('person_attributes', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class InsuredObjectAttributes(db.Model):
    __tablename__ = 'insured_object_attributes'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    object_type = db.Column(db.String(32))
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))
    attribute_3 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('object_attributes', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ImplementationAttributes(db.Model):
    __tablename__ = 'implementation_attributes'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('implementation_attributes', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )


#
# Product Line Attributes
#

class ProductLineLife(db.Model):
    __tablename__ = 'product_line_life'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))
    attribute_3 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('productline_attributes_life', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ProductLineHealth(db.Model):
    __tablename__ = 'product_line_health'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))
    attribute_3 = db.Column(db.String(32))
    attribute_4 = db.Column(db.String(32))
    attribute_5 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('productline_attributes_health', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ProductLinePnC(db.Model):
    __tablename__ = 'product_line_pnc'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('productline_attributes_pnc', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ProductLineCar(db.Model):
    __tablename__ = 'product_line_car'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('productline_attributes_car', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

#
# Insured Object Type Attributes
#

class ObjectHouse(db.Model):
    __tablename__ = 'object_house'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('object_attributes_house', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ObjectCar(db.Model):
    __tablename__ = 'object_car'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('object_attributes_car', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ObjectFactory(db.Model):
    __tablename__ = 'object_factory'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))
    attribute_3 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('object_attributes_factory', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ObjectField(db.Model):
    __tablename__ = 'object_field'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))
    attribute_2 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('object_attributes_field', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )

class ObjectForest(db.Model):
    __tablename__ = 'object_forest'
    id = db.Column(db.Integer, primary_key=True)
    activity_id = db.Column(db.LargeBinary, db.ForeignKey('activities.id'), nullable=False)
    attribute_1 = db.Column(db.String(32))

    # relationships
    activity = db.relationship(
        'Activity',
        backref=db.backref('object_attributes_forest', uselist=False),
        foreign_keys=[activity_id],
        uselist=False,
    )


from flask import jsonify, request, current_app
from datetime import date
from polzybackend.policy import Policy
from polzybackend.models import User, Activity, ActivityType
from polzybackend.utils.import_utils import get_policy_class, get_all_stages, get_activity_class
from polzybackend.main import bp
from polzybackend import auth
from polzybackend.GamificationHandler import update_achievement

@bp.route('/stage')
def get_stages():
    #
    # returns list of all available stages
    #

    try:
        # get all stages
        stages = get_all_stages()()
        current_app.logger.debug(f"Value of stages: {stages}")

    except Exception as e:
        current_app.logger.warning(f'Failed to get All Stages: {e}')
        stages = []

    return jsonify(stages), 200


@bp.route('/policy/<string:policy_number>/<string:effective_date>')
@bp.route('/policy/<string:policy_number>')
@auth.login_required
def get_policy(policy_number, effective_date=None):
    #
    # fetches a Policy by policy_number & effective_data
    # and returns it
    #

    # set default effective_date if needed
    if effective_date is None:
        effective_date = str(date.today())

    try:
        # get Policy
        policy = get_policy_class()(policy_number, effective_date)
        # update policy stage and language
        policy.setStage(auth.current_user().stage)
        policy.setLanguage(auth.current_user().language)
        current_app.logger.warning(f"Stage={policy.stage}, lang={policy.language}")

        if policy.fetch():
            policy.set_user(auth.current_user())
            current_app.config['POLICIES'][policy.uuid] = policy
            result = policy.get()
            # DEBUG
            #import json 
            #print(json.dumps(result, indent=2))

            # save activity to DB
            Activity.read_policy(policy_number, effective_date, auth.current_user())
            
            # set response
            response_code = 400 if 'error' in result else 200
            if response_code == 200:
                update_achievement()
            return jsonify(result), response_code


    except Exception as e:
        current_app.logger.exception(f'Fetch policy {policy_number} {effective_date} failed: {e}')
        return jsonify({'error': str(e)}), 400

    return jsonify({'error': 'Policy not found'}), 404


@bp.route(f'/activity', methods=['POST'])
@auth.login_required
def new_activity():
    #
    # create new activity
    #

    # get post data
    data = request.get_json()

    # get policy and create activity 
    try:
        # get policy from app store
        policy = current_app.config['POLICIES'].get(data['id'])
        if policy is None:
            raise Exception(f'Policy {data["id"]} is absent in PoLZy storage')

        # save activity to DB
        activity = Activity.new(data, policy, auth.current_user())

        # execute activity
        result = policy.executeActivity(data['activity'])
        # update activity status
        activity.finish('OK' if result else 'Failed')

        if result:
            # update policy
            policy.setStage(auth.current_user().stage)
            policy.setLanguage(auth.current_user().language)
            current_app.logger.warning(f"Stage={policy.stage}, lang={policy.language}")

            # check if activity returns not bool result
            if not result is True:
                update_achievement()
                return jsonify(result), 200

            policy.fetch()
            update_achievement()
            return jsonify(policy.get()), 200
        
    except Exception as e:
        current_app.logger.warning(f'Execution activity {data.get("name")} for policy {policy.policy_number} faild: {e}')
        return jsonify({'error': 'Bad Request'}), 400

    return jsonify({
        'id': str(activity.id),
        'status': 'Failed',
    }), 400



from . import bp
from polzybackend import messenger
from polzybackend.models import User
from flask import jsonify
import json


# toast debugging route
@bp.route('/ping')
def ping():
    #msg = messenger.format_sse(data='ping', event='fasifu')
    data = {
        'text': 'Fasifu message!',
        #'variant': 'success',
        #'anchorOrigin': {
        #    'vertical': 'top',
        #    'horizontal': 'left',
        #},
        #'autoHideDuration': 2000,
    }
    msg = f"data: {json.dumps(data)}\n\n"
    messenger.announce(msg=msg)
    return {}, 200


# returns all available users
@bp.route('/users')
def get_users():
    users = User.query.all()
    return jsonify([u.email for u in users]), 200



from polzybackend import db, auth
from polzybackend.utils.auth_utils import generate_id, generate_token, get_expired, is_supervisor, load_attributes
from datetime import datetime, timedelta, date
from sqlalchemy import and_
from functools import reduce

# system date format
date_format = "%Y-%m-%d"

# authentication
@auth.verify_token
def verify_token(token):
    user = User.query.filter_by(access_key=token).first()
    if user and user.key_expired > datetime.utcnow():
        return user


#
# Authentication Models
#

#
# Association Models
#

# association table for user-role relation
user_company_roles = db.Table(
    'user_company_roles',
    db.Column('user_id', db.String(56), db.ForeignKey('users.id'), primary_key=True),
    db.Column('company_id', db.String(56), db.ForeignKey('companies.id'), primary_key=True),
    db.Column('role_id', db.Integer, db.ForeignKey('roles.id'), primary_key=True)
)

class CompanyToCompany(db.Model):
    #
    # association object between companies
    #
    __tablename__ = 'company_company'
    parent_id = db.Column(db.String(56), db.ForeignKey('companies.id'), primary_key=True)
    child_id = db.Column(db.String(56), db.ForeignKey('companies.id'), primary_key=True)
    attributes = db.Column(db.String(1024), nullable=True)

    # relationships
    parent = db.relationship('Company', backref='child_companies', foreign_keys=[parent_id])
    child = db.relationship('Company', backref='parent_companies', foreign_keys=[child_id])

class UserToCompany(db.Model):
    #
    # association object between user and company
    #
    __tablename__ = 'user_company'
    user_id = db.Column(db.String(56), db.ForeignKey('users.id'), primary_key=True)
    company_id = db.Column(db.String(56), db.ForeignKey('companies.id'), primary_key=True)
    attributes = db.Column(db.String(1024), nullable=True)

    # relationships
    user = db.relationship('User', backref='companies', foreign_keys=[user_id])
    company = db.relationship('Company', backref='users', foreign_keys=[company_id])
    roles = db.relationship(
        'Role',
        secondary=user_company_roles,
        primaryjoin=and_(user_id == user_company_roles.c.user_id, company_id == user_company_roles.c.company_id),
        backref=db.backref('user_to_companies'),
    )

    def to_json(self):
        # JSON-string to dict 
        #load_json = lambda string: json.loads(string) if string else None
        return {
            'id': self.company.id,
            'name': self.company.name,
            'displayedName': str(self.company),
            #'roles': [role.name for role in self.roles],
            'attributes': load_attributes(self.company.attributes),
            #'userAttributes': load_json(self.attributes),
        }

    def to_admin_json(self):
        child_companies = CompanyToCompany.query.filter_by(parent_id=self.company_id).all()
        return {
            'id': self.company.id,
            'name': self.company.name,
            'displayedName': str(self.company),
            'childCompanies': [{
                'id': ch.child_id,
                'name': ch.child.name,
                'attributes': ch.attributes,
            } for ch in child_companies],
            'users': [{
                'id': u.user.id,
                'email': u.user.email,
                'roles': [r.name for r in u.roles],
                'attributes': u.attributes if u.attributes else '',
            } for u in self.company.users],
        }

class Role(db.Model):
    __tablename__ = 'roles'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(16), unique=True, nullable=False)
    is_supervisor = db.Column(db.Boolean, nullable=False, default=False)


class User(db.Model):
    __tablename__ = 'users'
    id = db.Column(db.String(56), primary_key=True, default=generate_id)
    email = db.Column(db.String(64), unique=True, nullable=False)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    displayed_name = db.Column(db.String(64), nullable=True)
    first_name = db.Column(db.String(64), nullable=True)
    last_name = db.Column(db.String(64), nullable=True)
    #oauth_provider_id = db.Column(db.Integer, db.ForeignKey('oauth_providers.id'), nullable=False)
    #oauth_user_id = db.Column(db.String(128), nullable=False)
    #oauth_token = db.Column(db.String(128), nullable=False)
    access_key = db.Column(db.String(128), nullable=False, default=generate_token)
    key_expired = db.Column(db.DateTime, nullable=False, default=get_expired)
    # current session attributes
    stage = db.Column(db.String(8), nullable=True)
    language = db.Column(db.String(8), nullable=True)
    company_id = db.Column(db.String(56), db.ForeignKey('companies.id'), nullable=True)

    # relationships
    #oauth_provider = db.relationship(
    #    'OAuthProvider',
    #    foreign_keys=[oauth_provider_id],
    #)
    company = db.relationship(
        'UserToCompany',
        primaryjoin=and_(
            id == UserToCompany.user_id,
            company_id == UserToCompany.company_id,
        ),
        uselist=False,
    )

    def __str__(self):
        return self.displayed_name or self.first_name or self.last_name or self.email.split('@')[0]

    def set_stage(self, stage):
        self.stage = stage
        db.session.commit()

    def set_language(self, language):
        self.language = language
        db.session.commit()

    def set_company(self, company_id=None, company=None):
        # check if either company data is provided
        if company_id is None and company is None:
            raise Exception('Neither Company ID nor Company provided')

        # check if company should be fetched
        if company is None:
            company = Company.query.get(company_id)
        
        # check if company exists
        if company is None:
            raise Exception('Company not found')

        # get UserToCompany instance
        user_company = UserToCompany.query.filter(and_(
            UserToCompany.user_id == self.id,
            UserToCompany.company_id == company.id,
        )).first()

        # check if association exists
        if user_company is None:
            raise Exception(f'User is not assigned to company ID {company_id}')

        # set company
        self.company_id = company.id
        db.session.commit()

        # return company json
        return user_company.to_json()

    def is_supervisor(self):
        return reduce(lambda result, company: result or is_supervisor(company), self.companies, False)

    def get_permissions(self):
        if self.company:
            user_roles = [r.name.lower() for r in self.company.roles]

            return {
                'policy': any(role in ['admin', 'clerk'] for role in user_roles),
                'antrag': any(role in ['admin', 'agent'] for role in user_roles),
            }

        return {
            'policy': False,
            'antrag': False,
        }

    def to_json(self):
        return {
            'name': str(self),
            'isSupervisor': self.is_supervisor(),
            'accessToken': self.access_key,
            'companies': [company.to_json() for company in self.companies],
        }

    def get_admin_json(self):
        return {
            'possibleRoles': [r.name for r in Role.query.all()],
            'companies': [company.to_admin_json() for company in filter(
                lambda company: is_supervisor(company),
                self.companies
            )],
        }

'''
class OAuthProvider(db.Model):
    __tablename__ = 'oauth_providers'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), unique=True, nullable=False)
    client_id = db.Column(db.String(128), nullable=False)
    secret_key = db.Column(db.String(128), nullable=False)

    def __str__(self):
        return self.name
'''

#
# Company Model
#
class Company(db.Model):
    __tablename__="companies"
    id = db.Column(db.String(56), primary_key=True, default=generate_id)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    name = db.Column(db.String(128), unique=True, nullable=False)
    displayed_name = db.Column(db.String(64), nullable=True)
    email = db.Column(db.String(32), nullable=True)
    phone = db.Column(db.String(16), nullable=True)
    country = db.Column(db.String(32), nullable=True)
    post_code = db.Column(db.String(8), nullable=True)
    city = db.Column(db.String(32), nullable=True)
    address = db.Column(db.String(64), nullable=True)
    attributes = db.Column(db.String(1024), nullable=True)

    def __str__(self):
        return self.displayed_name or self.name


#
# Activity Models
#
class Activity(db.Model):
    __tablename__ = 'activities'
    id = db.Column(db.String(56), primary_key=True, default=generate_id)
    creator_id = db.Column(db.String(56), db.ForeignKey('users.id'), nullable=False)
    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    policy_number = db.Column(db.String(64), nullable=False)
    effective_date = db.Column(db.Date, nullable=False, default=date.today)
    type = db.Column(db.String(32), nullable=False)
    status = db.Column(db.String(64), nullable=True)
    finished = db.Column(db.DateTime, nullable=True)
    attributes = db.Column(db.String, nullable=True)
    
    # relationships
    creator = db.relationship(
        'User',
        backref=db.backref('created_activities', order_by='desc(Activity.created)'),
        foreign_keys=[creator_id],
    )

    def __str__(self):
        return self.id

    @classmethod
    def new(cls, data, policy, current_user):
        # 
        # create instance using data
        #

        instance = cls(
            policy_number=policy.number,
            effective_date=datetime.strptime(policy.effective_date, date_format).date(),
            type=data['activity'].get('name'),
            creator=current_user,
            attributes=json.dumps(data['activity'].get('fields'))
        )
        
        # store to db
        db.session.add(instance)
        db.session.commit()
        
        return instance

    @classmethod
    def read_policy(cls, policy_number, effective_date, current_user):
        #
        # create instance of reading a policy
        #

        instance = cls(
            policy_number=policy_number,
            effective_date=datetime.strptime(effective_date, date_format).date(),
            type='Read Policy',
            creator=current_user,
            finished=datetime.utcnow(),
            status='OK',
        )

        # store to db
        db.session.add(instance)
        db.session.commit()
        
        return instance


    def finish(self, status):
        #
        # sets is_finished to True 
        #
        self.status = status
        self.finished = datetime.utcnow()
        db.session.commit()

class ActivityType(db.Model):
    __tablename__ = 'activity_types'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), nullable=False)
    description = db.Column(db.String(128), nullable=True)





from flask import jsonify, request, current_app
from datetime import date
from polzybackend.authenticate import bp
from polzybackend.models import User
from polzybackend import auth
from polzybackend.utils.import_utils import get_permissions


@bp.route('/login', methods=['POST'])
def login():
    # get request data
    data = request.get_json()
    
    # check for required fields in data
    required_fields = [
        'email',
        'stage',
        'language',
    ]
    for field in required_fields:
        value = data.get(field)  
        if value is None:
            return {'error': f'User data does not contain {field}'}, 400

    # get user from db
    email = data['email']
    user = User.query.filter_by(email=email).first()
    # check if user found
    if user is None:
        return {'error': f'User {email} does not exist'}, 404

    # update current stage and language
    user.set_stage(data['stage'])
    user.set_language(data['language'])

    # update company if user has only one
    #if len(user.companies) == 1:
    #    user.set_company(company=user.companies[0].company)

    return jsonify(user.to_json()), 200


@bp.route('/permissions', methods=['POST'])
@auth.login_required
def get_user_permissions():
    # get request data
    data = request.get_json()

    # get company id
    company_id = data.get('id')
    if company_id is None:
        return {'error': 'Company data does not contain company id'}, 400

    # update company
    user = auth.current_user()
    try:
        company_details = user.set_company(company_id=company_id)
    except Exception as e:
        return {'error': f'Set company failed: {e}'}, 400

    #return {'success': 'Success'}, 200
    return jsonify({
        'permissions': get_permissions(user),
        'company': company_details,
    }), 200
